<?php

namespace Drupal\critical_css\Asset;

use Drupal\Core\Asset\AssetCollectionRendererInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Decorates the CSS collection renderer service, adds Critical CSS.
 *
 * @see \Drupal\Core\Asset\CssCollectionRenderer
 */
class CssCollectionRenderer implements AssetCollectionRendererInterface {

  /**
   * The decorated CSS collection renderer.
   *
   * @var \Drupal\Core\Asset\AssetCollectionRendererInterface
   */
  protected $cssCollectionRenderer;

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Critical CSS provider.
   *
   * @var \Drupal\critical_css\Asset\CriticalCssProviderInterface
   */
  protected $criticalCssProvider;

  /**
   * Constructs a CssCollectionRenderer.
   *
   * @param \Drupal\Core\Asset\AssetCollectionRendererInterface $css_collection_renderer
   *   The decorated CSS collection renderer.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config Factory.
   * @param \Drupal\critical_css\Asset\CriticalCssProviderInterface $critical_css_provider
   *   The Critical CSS provider.
   */
  public function __construct(AssetCollectionRendererInterface $css_collection_renderer, ConfigFactoryInterface $config_factory, CriticalCssProviderInterface $critical_css_provider) {
    $this->cssCollectionRenderer = $css_collection_renderer;
    $this->configFactory = $config_factory;
    $this->criticalCssProvider = $critical_css_provider;
  }

  /**
   * {@inheritdoc}
   */
  public function render(array $css_assets) {
    $elements = $this->cssCollectionRenderer->render($css_assets);

    // Get critical CSS and change parent's output if needed.
    $criticalCss = $this->criticalCssProvider->getCriticalCss();
    if ($criticalCss) {
      $newElements = [];
      foreach ($elements as $element) {
        // Skip files with print media.
        if ($element['#attributes']['media'] == 'print') {
          $newElements[] = $element;
        }
        else {
          if ($this->configFactory->get('critical_css.settings')
            ->get('preload_non_critical_css')) {
            // Add a preload link so non-critical CSS is loaded with the highest
            // priority.
            $preloadElement = $element;
            $preloadElement['#attributes']['rel'] = 'preload';
            $preloadElement['#attributes']['as'] = 'style';
            unset($preloadElement['#attributes']['media']);
            $newElements[] = $preloadElement;
          }

          // Add a stylesheet link with print media and an "onload" event.
          // @see https://www.filamentgroup.com/lab/load-css-simpler/
          // TODO Find a better way to set the onload attribute:
          // Due to Drupal escaping quotes inside an attribute, we need to set a
          // dummy "data-onload-media" attribute, only needed for the onload
          // attribute.
          // "this.media='all'" gets escaped into
          // "this.media=&#039;all&#039;".
          $onLoadElement = $element;
          $onLoadElement['#attributes']['media'] = 'print';
          $onLoadElement['#attributes']['data-onload-media'] = 'all';
          $onLoadElement['#attributes']['onload'] = "this.onload=null;this.media=this.dataset.onloadMedia";
          $newElements[] = $onLoadElement;

          // Add fallback element for non-JS browsers.
          $noScriptElement = $element;
          $noScriptElement['#noscript'] = TRUE;
          $newElements[] = $noScriptElement;
        }
      }
      return $newElements;
    }
    return $elements;
  }

}
